/**
 * test-webid
 * Copyright (C) 2011 Thomas Bergwinkl
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package org.bergnet.test.webid.testcases;

import java.util.ArrayList;

import org.bergnet.test.EarlTestCase;
import org.bergnet.test.webid.WebIDTestException;
import org.bergnet.test.webid.WebIDTests;
import org.bergnet.test.webid.XsdAnyUriDatatype;
import org.bergnet.webid.commons.CryptoUtils;
import org.bergnet.webid.commons.FOAFBuilder;
import org.bergnet.webid.commons.WebID;
import org.junit.Assert;
import org.junit.Test;

import com.hp.hpl.jena.rdf.model.ResourceFactory;


public class MultipleURIs extends AbstractTestCase {
	private String[] ids; 
			
	public MultipleURIs() throws Exception {
		init();

		ids = new String[] {"multipleuris1.rdf", "multipleuris2.rdf"};
		verificationQuery = "PREFIX earl: <http://www.w3.org/ns/earl#>\n" +
				"PREFIX wit: <http://www.w3.org/2005/Incubator/webid/earl/RelyingParty#>\n" +
				"ASK WHERE {\n" +
				"	?webidClaimAssertion1\n" +
				"		earl:test wit:webidClaim;\n" +
				"		earl:result ?webidClaimResult1;\n" +
				"		earl:subject ?webidClaimSubject1.\n" +
				"	?webidClaimResult1 earl:outcome earl:passed.\n" +
				"	?webidClaimSubject1 wit:claimedIdentity ?webid1String.\n" +
				"	?webidClaimAssertion2\n" +
				"		earl:test wit:webidClaim;\n" +
				"		earl:result ?webidClaimResult2;\n" +
				"		earl:subject ?webidClaimSubject2.\n" +
				"	?webidClaimResult2 earl:outcome earl:passed.\n" +
				"	?webidClaimSubject2 wit:claimedIdentity ?webid2String.\n" +
				"	?webidAuthenticationAssertion\n" +
				"		earl:test wit:webidAuthentication;\n" +
				"		earl:result ?webidAuthenticationResult.\n" +
				"	?webidAuthenticationResult earl:outcome earl:passed }";

		verificationVariables.clear();
		verificationVariables.add("webid1String", ResourceFactory.createTypedLiteral(getFullUrl(ids[0] + fragment), XsdAnyUriDatatype.getInstance()));		
		verificationVariables.add("webid2String", ResourceFactory.createTypedLiteral(getFullUrl(ids[1] + fragment), XsdAnyUriDatatype.getInstance()));
	}

	public WebID generateWebID() throws WebIDTestException {		
		try {
			ArrayList<String> uriArray = new ArrayList<String>();

			for(String id : ids)
				uriArray.add(getFullUrl(id + "#me"));

			webid = WebID.generate(keySize, uriArray.toArray(new String[] {}),  WebID.createDeltaDate(-1), WebID.createDeltaDate(1));

			CryptoUtils.writePemFile(webid, getOutputFolder() + "/webid.pem");

			return webid;
		} catch(Exception e) {
			throw new WebIDTestException("Can't generate WebID", e);
		}
	}

	public void publishFOAF() throws WebIDTestException {
		try {
			for(String id : ids)
				WebIDTests.getInstance().getPublisher().publish(id, foaf, FOAFBuilder.FORMAT_RDF_XML);
		} catch(Exception e) {
			throw new WebIDTestException("Can't publish foaf file");
		}
	}

	@EarlTestCase(
			url="http://www.w3.org/2005/Incubator/webid/earl/TestSuite#multipleUris",
			title="Does the endpoint support multiple URIs?",
			description="Creates a WebID with multiple URIs and publishes FOAF files for all URIs.")
	@Test
	public void test() throws Exception {
		generateWebID();
		buildFOAF();
		publishFOAF();
		login();
		verifyOutput();

		Assert.assertTrue(result);
	}
}
