package org.bergnet.webid.commons;

import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.security.Key;
import java.security.KeyStore;
import java.security.PrivateKey;
import java.security.Security;
import java.security.cert.Certificate;
import java.security.cert.X509Certificate;

import org.bouncycastle.jce.provider.X509CertParser;
import org.bouncycastle.openssl.PEMWriter;

public class CryptoUtils {
	public static void writePemFile(WebID webid, String fileName) throws WebIDException {
		try {
			FileWriter fileWriter = new FileWriter(fileName);
			PEMWriter pemWriter = new PEMWriter(fileWriter);

			pemWriter.writeObject(webid.getCertificate());
			pemWriter.writeObject(webid.getPrivateKey());

			pemWriter.close();
			fileWriter.close();
		} catch(Exception e) {
			throw new WebIDException("Can't write WebID to PEM file", e);
		}
	}

	public static void writePkcs12File(WebID webid, String fileName, String alias, String password) throws WebIDException {
		try {
			KeyStore.Builder ksBuilder = KeyStore.Builder.newInstance("PKCS12", Security.getProvider("BC"), new KeyStore.PasswordProtection(password.toCharArray()));
			KeyStore ks = ksBuilder.getKeyStore();

			ks.setKeyEntry(alias, webid.getPrivateKey(), password.toCharArray(), new Certificate[] { webid.getCertificate() });

			FileOutputStream fileOutputStream = new FileOutputStream(fileName);
			ks.store(fileOutputStream, password.toCharArray());
			fileOutputStream.close();
		} catch(Exception e) {
			throw new WebIDException("Can't write WebID to PKCS12 file", e);
		}
	}

	public static WebID readPkcs12File(String fileName, String alias, String password) throws WebIDException {
		try {
			KeyStore ks = KeyStore.getInstance("PKCS12");

			FileInputStream fileInputStream = new FileInputStream(fileName);
			ks.load(fileInputStream, password.toCharArray());
			fileInputStream.close();

			if(alias == null) {
				if(ks.aliases().hasMoreElements())
					alias = ks.aliases().nextElement();
				else
					throw new WebIDException("Can't find an alias");
			}

			Key key = ks.getKey(alias, password.toCharArray());
			Certificate certificate = ks.getCertificate(alias);

			if(key instanceof PrivateKey && certificate instanceof X509Certificate)
				return new WebID((PrivateKey)key, (X509Certificate)certificate);
			else
				return null;
		} catch(Exception e) {
			throw new WebIDException("Can't read WebID from PKCS12 file", e);
		}
	}

	public static X509Certificate readX509File(String fileName) throws WebIDException {
		try {
			X509CertParser certParser = new X509CertParser();
			certParser.engineInit(new FileInputStream(fileName));

			Object X509Object = certParser.engineRead();

			if(X509Object instanceof X509Certificate)
				return (X509Certificate)X509Object;
			else
				throw new WebIDException("File doesn't contain a X509 certificate");
		} catch(Exception e) {
			throw new WebIDException("Can't read X509 certificate from file", e);
		}
	}
}
